VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "PPpage"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'CLASS MODULE TO CREATE A PAGE FOR PRINTING OR PREVIEW.

'There should be nothing here that is specific to any particular project except that the
'error routines require a Public Boolean, 'canceled', in the main program.

' Philip Bolt, 2005, 2006, 2007
'Last updated: 10/3/7

Option Explicit
Option Compare Text

Const b_wt As Integer = 700         'Font BOLD weight value.
Const n_wt As Integer = 400         'Font NORMAL weight value.
Const fl_char As String = "\"       'Text formating LEFT character.
Const fr_char As String = "/"       'Text formating RIGHT character.
Const form_sd As String = "dd/mm/yy" 'Short date format string.

'Page numbers.
Private p_l_num As Integer          'The page LOCAL number.
Private p_a_num As Integer          'The page ABSOLUTE number.

'Page structure values.
Private t_marg As Single            'Top margin.
Private h_marg As Single            'Header margin.
Private b_marg As Single            'Bottom margin.
Private f_marg As Single            'Footer margin.
Private l_marg As Single            'Left margin.
Private r_marg As Single            'Right margin.
Private max_p_l As Single           'Maximum page length, excluding bottom margins.
Private l_width As Single           'Line width, excluding margins.
Private p_inc As Integer            'Page increment, 1 for ALL, 2 for ODD/EVEN.
Private LRgap As Single             'Printer width dead spaces.
Private TBgap As Single
Private ratio As Single             'Print/Preview scale factor.
Private p_col As Long               'Paper colour.
 
'Font data values.
Private l_ht As Single              'Line height, (height of an 'I').
Private n_l_h As Single             'New line height, (max. 'l_ht' in line).
Private f_name As String            'Font name.
Private sym_font As String          'Symbol' font name.
Private wd_font As String           'Windings' font name.
Private f_size As Single            'Font size.
Private f_bold As Boolean           'Bold.
Private f_italic As Boolean         'Italics.
Private f_underline As Boolean      'Underline.
Private f_strikethrough As Boolean  'Strikethrough.
Private f_colour As Long            'Colour, (can only be applied as a FORECOLOR).
Private f_weight As String          'Weight.

Private s_f_n As String             'Stored font name.
Private s_f_z As Single             'Stored font size.
Private s_f_b As Boolean            'Stored font Bold flag.
Private s_f_i As Boolean            'Stored font Italic flag.
Private s_f_u As Boolean            'Stored font Underline flag.
Private s_f_s As Boolean            'Stored font StrikeThrough flag.
Private s_f_c As Long               'Stored font colour, (Forecolor),
Private ss_set As Boolean           'Super or Subscript set flag.

'Print/Preview values.
Private pp_flg As Integer           'PRINT/PREVIEW flag, 0 = Print, 1 = Preview.
Private print_depth As Single       'The printing depth on the page.
Private prntd As Single             '(temporary printing depth storage).
Private p_zoom As Single            'The current zoom factor.

'Formated text values.
Private coms As Integer             'Command count.
Private part_line As Single         'Length of the existing part of a line.
Private align As Integer            'Default alignment code.
Private crit_len As Single          'Critical length for a line.
Private spc_sz As Single            'Size of a space in the current font.
Private com() As String             'Format command string array.
Private t_char As String            'Current character.
Private ccw As Single               'Current character size, (for backspacing).
Private b_s_f As Boolean            'Backspace flag.
Private check As Boolean            'Line size checking flag.

Private g_obj As Object             'Global object for display.

'Graphics values.
Private pic As Object               'Graphics picture box.

Private page_exists As Boolean      'Page created flag.
Private setup As Boolean            'Object set up flag.

Private frm As Object               'Form object.
Private f_pic As Object             'Preview Form text picture object.

'Text alignment constants.
'VERTICAL
Const TA_TOP = 0            'The reference point will be on the top edge of the bounding
                            'rectangle of the text.
Const TA_BASELINE = 24      'The reference point will be on the baseline of the text.
Const TA_BOTTOM = 8         'The reference point will be on the bottom edge of the bounding
                            'rectangle of the text.
'HORIZONTAL
Const TA_LEFT = 0           'The reference point will be on the left edge of the bounding
                            'rectangle of the text.
Const TA_RIGHT = 2          'The reference point will be on the right edge of the bounding
                            'rectangle of the text.
Const TA_CENTER = 6         'The reference point will be horizontally centered along the
                            'bounding rectangle of the text.
'OTHER
Const TA_NOUPDATECP = 0     'Do not set the current point to the reference point.

Const TA_RTLREADING = 256   'Win 95/98 only:Display the text right-to-left (if the font is
                            'designed for right-to-left reading).
Const TA_UPDATECP = 1       'Set the current point to the reference point.

'Set text alignment API procedure.
Private Declare Sub SetTextAlign Lib "gdi32.dll" (ByVal hDC As Long, _
                                           ByVal wFlags As Long)
'Get Text alignment API function.
Private Declare Function GetTextAlign Lib "gdi32.dll" (ByVal hDC As Long) As Long

'Printer font transparency data.
Private iBKMode As Long
Private Const TRANSPARENT = 1
Private Const OPAQUE = 2

'Printer font transparency API procedure.
Private Declare Function SetBkMode Lib "gdi32" (ByVal hDC As Long, ByVal nBkMode As Long) As Long

'Object constructor.
'Run when the object is created.

Private Sub Class_Initialize()

    If Not setup Then
        set_up
        setup = True
    End If
    
End Sub

'Procedure name:    Class_Terminate()
'Run when a page is terminated.

Private Sub Class_Terminate()

End Sub

'Routine to set up data when a Page object is created.
'Procedure name:    set_up

Private Sub set_up()
 
    sym_font = "Symbol"         'Store the 'Symbol' font name.
    wd_font = "WingDings"       'Store the 'WingDings' font name.
    
    pp_flg = 1                  'Default to PREVIEW and
    p_zoom = 1                  'zoom off.
    
    part_line = 0               'Start with no part_lines.
    align = 0                   'Default alignment LEFT.

    Set frm = preview           'Define the form object,
    Set f_pic = frm.Picture2    'the Text Picture Box object,
    Set pic = frm.graphic_box   'the graphic picture object.

    Set g_obj = f_pic           'Make the Preview Box the default global object.
    
    n_l_h = Printer.TextHeight("I") 'Initialise the LINE HEIGHT and
    spc_sz = Printer.TextWidth(" ") 'SPACE WIDTH.
    
    page_exists = True          'Flag that the page exists and
    ss_set = False              'clear the Super/Subscript and
    
End Sub

'Routine to change the default alignment to a new global value.
'This can will be over-ridden by actual values used in 'add_f_text' or 'add_text'.
'Procedure name:    set_align(a_v)
'Paramater:         a_v;    the alignment value.

Public Sub set_align(a_v)

    If a_v >= 0 And a_v <= 3 Then
        align = a_v
    Else
        MsgBox "Alignment codes must be in the range 0 to 3", vbOKOnly + vbInformation, "ALIGNMENT ERROR"
        canceled = True
    End If

End Sub

'Routine to read the default font data from the PPpages object.
'N.B. this isn't needed for programming.
'Procedure name:    font_defaults(f_ar())
'Parameter:         f_ar(); the font data array in the PPpages object.

Public Sub font_defaults(f_ar() As Variant)

   f_name = f_ar(0)
   f_size = f_ar(1)
   f_bold = f_ar(2)
   f_italic = f_ar(3)
   f_underline = f_ar(4)
   f_strikethrough = f_ar(5)
   f_colour = f_ar(6)
   f_weight = f_ar(7)
 
   Printer.ForeColor = f_colour
   g_obj.ForeColor = f_colour
 
End Sub

'Routine to set the page structure properties.
'Property Let name: set_sizes(opt as Integer, vlu as Variant)
'Parameter:         opt;    option number,
'                   vlu;    value.

Public Sub set_sizes(opt As Integer, vlu As Variant)

    Select Case opt
        Case 0
            max_p_l = vlu   'Maximum page length
        Case 1
            l_width = vlu   'Line width
        Case 2
            t_marg = vlu    'top margin.
        Case 3
            h_marg = vlu    'header margin,
        Case 4
            b_marg = vlu    'bottom margin.
        Case 5
            f_marg = vlu    'footer margin.
        Case 6
            l_marg = vlu    'left margin.
        Case 7
            r_marg = vlu    'right margin.
        Case 8
            p_inc = vlu     'page increment.
        Case 9
            LRgap = vlu     'Printer page dead width space.
        Case 10
            TBgap = vlu
        Case 11
            p_col = vlu     'Paper colour.
        Case 12
            ratio = vlu     'Print/Preview factor,
        Case Else
            MsgBox "Options must be in the range 0 to 12", vbOKOnly + vbCritical, "'Set_sizes' ERROR"
            canceled = True
    End Select
    
End Sub

'Routine to set the LOCAL and ABSOLUTE page numbers for the page.
'Procedure rather than Property Let used to allow optional parameter.

'Procedure name: page_numbers(lpn as Integer, Optional apn as Integer)
'Paramters:         lpn;    the LOCAL page number,
'                   apn;    the ABSOLUTE page number. This is optional. If missing. then
'                                                     defaults to the LOCAL.

Public Sub page_numbers(lpn As Integer, Optional apn As Integer = 0)
 
    p_l_num = lpn           'Assign the LOCAL page number

    If apn = 0 Then 'the ABSOLUTE page number is
        p_a_num = lpn           'either  a copy of the Local
    Else                        'or
        p_a_num = apn           'the parameter value.
    End If
 
End Sub

'Routine to read the page numbers of a specific page.
'Property name: page_number(opt As Integer)
'Parameter:     opt;    page selection option, 0 = LOCAL, 1 = ABSOLUTE.

Public Property Get page_number(opt As Integer)

    Select Case opt
        Case 0
            page_number = p_l_num 'LOCAL page number.
        Case 1
            page_number = p_a_num 'ABSOLUTE page number.
    End Select
    
End Property

'Routine to read the flag of a specific page.
'Property name: page_flag()

Public Property Get page_flag()

    page_flag = pp_flg  'The PRINT/PREVIEW flag.

End Property

'Set of property routines to allow access to page structure data.
'This is similar to the 'PP_pages' routine which should be used.
 
'Property name: page_data(opt)
'Parameter:     opt;    the item selection option.

Public Property Get page_data(opt As Integer)
 
    Select Case opt
        Case 0
            page_data = max_p_l 'Maximum page length
        Case 1
            page_data = l_width 'Line width
        Case 2
            page_data = t_marg  'top margin.
        Case 3
            page_data = h_marg  'header margin,
        Case 4
            page_data = b_marg  'bottom margin.
        Case 5
            page_data = f_marg  'footer margin.
        Case 6
            page_data = l_marg  'left margin.
        Case 7
            page_data = r_marg  'right margin.
        Case 8
            page_data = ratio   'Print/Preview scale factor.
        Case 9
            page_data = LRgap   'Printer Width dead space.
        Case 10
            page_data = TBgap   'Printer Width dead space.
        Case 11
            page_data = p_zoom  'The zoom factor.
        Case 12
            page_data = p_col   'The paper colour.
        Case Else
            MsgBox "Options must be in the range 0 to 11", vbOKOnly + vbCritical, "'PPpage:page_data' ERROR"
            canceled = True
    End Select
    
End Property

'Routine to set the ZOOM factor for a page.
'Property name: set_zoom(zmf As Single)
'Parameter:     zmf;    the zoom factor.

Public Property Let set_zoom(zmf As Single)
 
    If pp_flg = 1 Then  'Set the zoom if Preview.
        p_zoom = zmf
    Else                'Otherwise,
        p_zoom = 1      'make it unity.
    End If
 
End Property

'Routine to set the page font data.
'All parameters are optional so that any one can be set alone.
'In the absence of a value, the current value is retained.
'Commas' are required to indicate missed parameters but don't use trailing ones.
'Variants are used to allow the use of IsMissing.
'This routine allows the setting of font data on INDIVIDUAL PAGES rather than the
'font_setup' option which sets the font data for ALL pages before they're created.

'Procedure name:    font(Optional fnm As String = "", Optional fpt As Single,
'                        Optional fb As Boolean, Optional fi As Boolean,
'                        Optional fu As Boolean, Optional fs As Boolean,
'                        Optional fc As Long, Optional fw As Integer)

'Parameters:        fnm;    the font name,
'                   fpt;    the point size,
'                   fb;     the Bold flag,
'                   fi;     the Italic flag,
'                   fu;     the Undeline flag,
'                   fs;     the StrikeThrough flag,
'                   fc;     the RGB colour,
'                   fw;     the Weight, (using LOGFONT values),

'NB few fonts will accept weights other than 400, (Normal) or 700, (Bold).
'
Public Sub font(Optional fnm As String = "", Optional fpt As Single = 12, _
                Optional fb As Variant, Optional fi As Variant, Optional fu As Variant, _
                Optional fs As Variant, Optional fc As Variant, Optional fw As Variant)
 
    'STORE ANY CHANGES IN PARAMETERS
    If fnm <> "" Then
        f_name = fnm                'Set the font NAME,
    End If
        
    If IsNumeric(fpt) And fpt > 6 Then  'Don't accept a point size smaller than 6.
        f_size = fpt                'the SIZE
    End If
        
    If Not IsMissing(fb) Then
        f_bold = CBool(fb)          'BOLD,
        If f_bold Then
            f_weight = b_wt     'Set the WEIGHT to
        Else                    'match the BOLD flag,
            f_weight = n_wt
        End If
    End If
        
    If Not IsMissing(fi) Then
        f_italic = CBool(fi)        'ITALIC,
    End If
        
    If Not IsMissing(fu) Then
        f_underline = CBool(fu)     'UNDERLINE,
    End If
        
    If Not IsMissing(fs) Then
        f_strikethrough = CBool(fs) 'STRIKETHROUGH.
    End If
    
    If Not IsMissing(fc) Then
        f_colour = CLng(fc)         'COLOUR.
        g_obj.ForeColor = f_colour      'Set the text and
        Printer.ForeColor = f_colour    'printer colour.
    End If
    
    If Not IsMissing(fw) Then
        f_weight = CInt(fw)         'WEIGHT.
        If f_weight \ (n_wt + 1) > 0 Then 'this is BOLD or above.
            f_bold = True
        Else
            f_bold = False
        End If
    End If
   
    pp_font     'Re-set the font parameters.
    
End Sub

'Routine to set the font parameters in the light of the 'pp_flg' value.
'Both the PRINTER and Picture Box fonts are upgraded.
'Procedure name:    pp_font(objt)

Private Sub pp_font()
 
    Dim chp As Integer
 
    With Printer.font   'Set the printer font parameters.
        .Name = f_name
        .Size = f_size
        .Bold = f_bold
        .Italic = f_italic
        .Underline = f_underline
        .Strikethrough = f_strikethrough
        .Weight = f_weight
    End With
    
    l_ht = Printer.TextHeight("I")  'Store the height of a character, (line width), and
    spc_sz = Printer.TextWidth(" ")  'a space in the current font size.
 
    With g_obj.font 'Set the display object font parameters.
        .Name = f_name
        .Size = f_size * ratio * p_zoom
        .Bold = f_bold
        .Italic = f_italic
        .Underline = f_underline
        .Strikethrough = f_strikethrough
        .Weight = f_weight
    End With
    
    If Not check Then   'restore the text colour if this isn't a checking situation.
        Printer.ForeColor = f_colour
        g_obj.ForeColor = f_colour
    End If
 
End Sub

'Routine to return the current line height.
'Property name: line_height

Public Property Get line_height()

    line_height = n_l_h  'Return the current new line height.
    
End Property

'The item required is selected by the 'opt' parameter.
'Property name: font_data(opt)
'Parameter:     opt;    the item selection option.

Public Property Get font_data(opt As Integer)

    Select Case opt
        Case 0
            font_data = f_name          'Name,
        Case 1
            font_data = f_size          'point size,
        Case 2
            font_data = f_bold          'Bold
        Case 3
            font_data = f_italic        'Italic,
        Case 4
            font_data = f_underline     'Underline,
        Case 5
            font_data = f_strikethrough 'Strikethrough.
        Case 6
            font_data = f_colour        'Colour, (Forecolor).
        Case 7
            font_data = f_weight        'Weight, (for completeness; not actually needed).
        Case Else
            MsgBox "Options must be in the range 0 to 7", vbOKOnly + vbCritical, "'font_data' ERROR"
            canceled = True
    End Select
    
End Property

'Routine to set a SINGLE item of font data.
'Method name:   set_font(itm As Variant, opt As Integer)
'Parameter:     itm;    the item value,
'               opt;    the option number.


Public Sub set_font(itm As Variant, opt As Integer)

    Select Case opt
        Case 0
            f_name = CStr(itm)              'Name,
            Printer.font.Name = f_name
            g_obj.font.Name = f_name
        Case 1
            f_size = CSng(itm)              'point size,
            Printer.font.Size = f_size
            g_obj.font.Size = f_size
        Case 2
            f_bold = CBool(itm)             'Bold
            Printer.font.Bold = f_bold
            g_obj.font.Bold = f_bold
            If f_bold Then
                f_weight = b_wt     'Set the WEIGHT to
            Else                    'match the BOLD flag,
                f_weight = n_wt
            End If
            Printer.font.Weight = f_weight
            g_obj.font.Weight = f_weight
        Case 3
            f_italic = CBool(itm)           'Italic,
            Printer.font.Italic = f_italic
            g_obj.font.Italic = f_italic
        Case 4
            f_underline = CBool(itm)        'Underline,
            Printer.font.Underline = f_underline
            g_obj.font.Underline = f_underline
        Case 5
            f_strikethrough = CBool(itm)    'Strikethrough.
            Printer.font.Strikethrough = f_strikethrough
            g_obj.font.Strikethrough = f_strikethrough
        Case 6
            f_colour = CLng(itm)            'Colour, (Forecolor).
            Printer.ForeColor = f_colour    'Set the Printer
            g_obj.ForeColor = f_colour      'and display object text colour.
        Case 7
            f_weight = CInt(itm)            'Weight, (for completeness; not actually needed).
            If f_weight \ (n_wt + 1) > 0 Then 'this is BOLD or above.
                f_bold = True
            Else
                f_bold = False
            End If
            Printer.font.Bold = f_bold
            g_obj.font.Bold = f_bold
            Printer.font.Weight = f_weight
            g_obj.font.Weight = f_weight
        Case Else
            MsgBox "Options must be in the range 0 to 7", vbOKOnly + vbCritical, "'font_data' ERROR"
            canceled = True
    End Select

End Sub

'TEXT ADDITION ROUTINES.

'The PRINT/PREVIEW flag, 'pp_flg', is stored for the whole page.

'Routine to set the PRINT/PREVIEW flag for a page.
'This will CLEAR any data on the page to avoid the possibility of mixed
'Print and Preview formatted text.

'Property name: set_pp(ppf As Integer)
'Parameter:     ppf;    the PRINT/PREVIEW flag.

Public Property Let set_pp(ppf As Integer)

    If pp_flg = 0 Then  'Delete any existing text.
        Printer.KillDoc
    Else
        f_pic.Cls
    End If
    
    pp_flg = ppf    'Store the new value and
    pp_font         're-set the font parameters.
    
    print_depth = t_marg
    
End Property

'Routine to add SINGLE LINES of text to the printing area.
'The printing depth is returned to the calling routine via 'p_d'.
'The vertical position, 'vp', is an optional offset from the printing depth, 'p_d', which
'acts as a variable line width control. If ommmited, it is set to zero, (as will usually
'be the case).
'The alignment, 'alg', is also optional. If ommited, it is set to 0 = Left.

'If a line is too long for the page width and error message is displayed and the
'procedure ends.

'Procedure name:    add_text(tx, hp, tm, ByRef p_d, Optional vp, Optional alg)
'Parameters:        tx;     the TEXT,
'                   hp;     the HORIZONTAL position,
'                   tm;     the LINE TERMINATOR flag.
'                   p_d;    the print depth, which is updated and returned.
'                   alg;    the alignment, 0 = Left, 1 = Centre, 2 = Right, defaults to -1, (global value),
'                   vp;     the VERTICAL offset in twips, defaults to zero.

Public Sub add_text(tx, ByRef hp, tm, ByRef p_d, Optional alg = -1, Optional vp = 0)
 
    Dim psm As Integer
    Dim p_txt As Single
    Dim hpp As Single

    'Set the text display to LEFT justify and aligned with the BASE of letters.
    SetTextAlign g_obj.hDC, TA_LEFT Or TA_BASELINE
    SetTextAlign Printer.hDC, TA_LEFT Or TA_BASELINE

    p_txt = Printer.TextWidth(tx)
 
    print_depth = p_d   'Take the current printing depth from the parameter.

    If alg = -1 Then    'This is the defsult, so
        alg = align     'set the alignment to the current global value.
        If alg = 3 Then 'However, if this is set for FULL JUSTIFY, issue an error message.
            MsgBox "The global alignment of '3' can't be used.", vbOKOnly + vbCritical, "'add_text' ERROR"
            canceled = True
        End If
    End If
    
    If Not canceled Then '(1)
        If Not tm Then  'If the TERMINATION flag is cleared
            alg = 0     'only LEFT justification can be used.
        End If

        If hp = 0 Then  'Set the initial alignment.
            Select Case alg                 'All alignments include the left margin.
                Case 0 'is LEFT aligned.
                    hpp = hp + l_marg
                Case 1 'is CENTRE aligned.
                    hpp = l_marg + centre(tx) + hp
                Case 2 'is RIGHT aligned.
                    hpp = l_marg + l_width - p_txt - hp
                Case Else 'is an error, (e1).
                    MsgBox "Alignment codes must be 0, 1 or 2", vbOKOnly + vbCritical, "'add_text' ERROR"
                    canceled = True
            End Select
        Else
            hpp = hp
        End If
    
        If pp_flg = 0 Then
            hpp = hpp - LRgap                   'Modify the starting values.
            print_depth = print_depth - TBgap
        End If
 
        If Not canceled Then '(2)
            If p_txt + hpp > l_marg + l_width Then 'Text runs over the end of the line, (e2).
                MsgBox "The text is longer than a LINE.", vbOKOnly + vbCritical, "'add_text' ERROR"
            End If
 
            If pp_flg = 0 Then 'print.
                'Make the Printer font transparent.
                Printer.FontTransparent = True  'This is not sufficient,it also requires
                iBKMode = SetBkMode(Printer.hDC, TRANSPARENT)   'the API call.
                Printer.CurrentX = hpp              'Position text at specified X position and
                Printer.CurrentY = print_depth + vp 'Y position but
                Printer.Print tx;                   'don't terminate the print line.
            Else    'Preview
                g_obj.CurrentX = hpp                'Position text at specified X position and
                g_obj.CurrentY = print_depth + vp   'Y position but
                g_obj.Print tx;                     'don't terminate the print line.
            End If
 
            If tm Then
                terminate  'Terminate the printing line.
            Else
                If pp_flg = 0 Then
                    hp = Printer.CurrentX   'Return the horizontal position
                Else                        'via the parameter.
                    hp = g_obj.CurrentX
                End If
            End If
 
            p_d = print_depth   'Return the current print depth via the parameter.
        End If 'not CANCELed 2.
    End If 'not CANCELed 1.
    
End Sub

'Routine to add FORMATTED text to the printing area.
'The updated printing depth is returned to the calling routine via 'p_d'. If the default
'is used, the printing depth is set to the current value.
'The termination flag defaults to False which will return a partial line value for
'the last line of the text.
'The alignment, 'alg', is also optional. If ommited, it is set to -1 = use global value.
'The vertical position, 'vp', is an optional offset from the printing depth, 'p_d', which
'acts as a variable line width control. If ommmited, it is set to zero,
'hp is a horizontal offset which reduces the width of the line. i.e. an indent.

'This is NOT a word processor.
'Text already displayed is NOT modified by the addition of extra text. e.g. changing the
'margins doesn't effect the existing display.

'Procedure name:    add_f_text(tx, ByRef p_d, Optional tm As Boolean = False, Optional alg = -1,
'                              Optional p_l As Single = 0, Optional vp As Single = 0,
'                              Optional hp As Single = 0)

'Parameters:        tx;     the TEXT,
'                   p_d;    the PRINT DEPTH, default = 0, (existing value),
'                   tm;     the LINE TERMINATOR flag, default False,
'                   alg;    the ALIGNMENT, 0 = Left, 1 = Centre, 2 = Right, 3 = Justified,
'                           optional, default = -1, (global value),
'                   p_l;    the PART LINE length, optional, default = none, (0),
'                   vp;     the VERTICAL position in units, (this is an offset for EACH
'                           line), optional; default = 0, standard line spacing,
'                   hp;     the HORIZONTAL offset in units, default none (0),
'                           NB negative values for 'h_p' give both LEFT & RIGHT indents.

Public Sub add_f_text(tx, Optional p_d = 0, Optional tm As Boolean = False, Optional alg = -1, _
                      Optional p_l As Single = 0, Optional vp As Single = 0, _
                      Optional h_p As Single = 0)

    Dim p_txt, j, sect, spac, char, st_sc, st_ch As Integer
    Dim l_spac(3), c_h_p(1), a_spac, hpp, hph, vpv, hp As Single
    Dim done As Boolean
 
    'Set the text display to align with the LEFT and the BASE of letter bounding boxes.
    SetTextAlign g_obj.hDC, TA_LEFT Or TA_BASELINE
    SetTextAlign Printer.hDC, TA_LEFT Or TA_BASELINE
 
    If alg > 3 Or alg < -1 Then
        MsgBox "Alignment codes must be 0, 1, 2 or 3", vbOKOnly + vbCritical, "'add_f_text' ERROR"
        canceled = True
    End If

    If Not canceled Then
    
        hp = Abs(h_p)   'Store the operational value for the INDENT.

        re_set_font     'Reset the font to clear any previous formating.
        store_format    'Store the initial formating as a default for the first line.
        
        done = False    'Clear the Done flag.
        
        If alg = -1 Then    'This is the default value, so
            alg = align     'set the alignment to the current default.
        End If

        If p_d <> 0 Then
            print_depth = p_d + vp          'Take the current printing depth from the parameters.
        Else                                'or
            print_depth = print_depth + vp  'use the existing value.
        End If
        
        If vp = 0 Then      'If there is no added vertical space,
            part_line = p_l 'set the partial line value to the parameter.
        Else                'Otherwise,
            part_line = 0   'set the partial line value to zero.
        End If
  
        If h_p >= 0 Then 'set 'max. line length without 'hp'
            crit_len = l_marg + l_width     'Store the maximum length of a line.
        Else    'Otherwise, deduct 'hp' for a RIGHT indent as well as LEFT.
            crit_len = l_marg + l_width - hp
        End If
        
        If l_marg + hp > crit_len - part_line Then 'There isn't enough space left on this line, so
            terminate       'terminate the line and
            part_line = 0   'reset the part line value.
        End If
        
        parse_text tx       'Split the text into its command and text sections.
        
        If pp_flg = 0 Then
            print_depth = print_depth - TBgap   'Modify the initial Y for PRINTing.
        End If
       
       'Initialise the SECTION, SPACE, CHARACTER and storage variables.
        sect = 0: l_spac(0) = 0: l_spac(1) = 0: l_spac(2) = 0: l_spac(3) = 0
        char = 1: st_sc = sect: st_ch = char: a_spac = 0
 
    'START OF THE MAJOR LINE LOOP.
        Do Until done
            sect = st_sc: char = st_ch: spac = 0: c_h_p(0) = l_marg + hp + part_line
            c_h_p(1) = l_marg + hp + part_line
 
            re_store_format 'Restore the formating at the start of the CHECK line.
 
    'FIND THE TEXT FOR ONE LINE.
            If com(0, sect) <> "" Then 'Set up the initial formating.
                form_font com(0, sect)
            End If
 
            char = clear_spaces(char, com(1, sect))
 
            n_l_h = 0   'Initialise the new line height.
            Do Until (c_h_p(0) > crit_len) Or done  'loop until a line is completed.
                t_char = Mid(com(1, sect), char, 1) 'Store the character.
 
                If t_char = " " Then 'this is a space, so
                    spac = spac + 1         'COUNT the space and
                    l_spac(0) = sect        'store its SECTION,
                    l_spac(1) = char        'POSITION and
                    l_spac(2) = c_h_p(0)    'CURRENT LINE LENGTHS before adding the
                    l_spac(3) = c_h_p(1)    'space.
                End If
 
                check_text t_char, c_h_p(0), c_h_p(1)   'Find the lengths of this character and
                char = char + 1                         'count it.
            
                If char > Len(com(1, sect)) Then 'this is the end of this block, so
                    sect = sect + 1             'move to the next section.
                    If sect <= coms Then        'If there are more sections,
                        form_font com(0, sect)  'set the formatting and
                        char = 1                'reset the character position.
                    Else                        'Otherwise,
                        l_spac(0) = sect - 1    'store the last SECTION,
                        l_spac(1) = char - 1    'POSITION and
                        l_spac(2) = c_h_p(0)    'CURRENT LINE LENGTHS, PRINT and
                        l_spac(3) = c_h_p(1)    'PREVIEW, of the final part.
                        done = True
                    End If
                End If 'this is the end of a block.
            Loop 'until a line is checked.
 
            If spac = 0 And Not done Then 'a complete word wouldn't fit in the space, so
                terminate       'terminate this line and
                part_line = 0   'reset the part line value.
            Else
                If l_spac(1) = 1 And t_char = " " Then 'this is the first space in a new section, so
                    l_spac(0) = l_spac(0) - 1           'decrement the section number,
                    l_spac(1) = Len(com(1, l_spac(0)))  'set the character to the last position and
                    l_spac(2) = l_spac(2) - spc_sz      'deduct a space from the length.
                    l_spac(3) = l_spac(3) - spc_sz
                End If
        
                'Remove any trailing spaces from the line.
                If Mid(com(1, l_spac(0)), l_spac(1), 1) = " " And l_spac(1) > 1 Then
                    Do Until Mid(com(1, l_spac(0)), l_spac(1), 1) <> " "
                        l_spac(1) = l_spac(1) - 1
                    Loop
                End If

                If Not done Then
                    spac = spac - 1 'Decrement the space count to discount the final space.
                End If
 'If done Then MsgBox l_spac(2) & "/" & l_spac(3)
 
    'PRINT/PREVIEW THE ACTUAL LINE.
                a_spac = 0      'Zero the additional space value.
                If pp_flg = 0 Then 'PRINT
                    hph = aline(alg, hp, l_spac(2) - l_marg, part_line) 'Set the horizontal value for the required alignment.
                    'Calculate the extra space to be added to each word for full justification.
                    If alg = 3 And Not done Then
                        If spac > 0 Then
                            a_spac = (crit_len - l_spac(2)) / spac  'Calculate the extra space.
                        Else
                            a_spac = crit_len - l_spac(2)
                        End If
                    End If
                Else    'PREVIEW
                    hph = aline(alg, hp, l_spac(3) - l_marg, part_line) 'Set the horizontal value for the required alignment.
                    'Calculate the extra space to be added to each word for full justification.
                    If alg = 3 And Not done Then
                        If spac > 0 Then
                            a_spac = (crit_len - l_spac(3)) / spac  'Calculate the extra space.
                        Else
                            a_spac = crit_len - l_spac(3)
                        End If
                    End If
                End If 'PRINT/PREVIEW

                sect = st_sc: char = st_ch: c_h_p(0) = hph: done = False
                
                re_store_format 'Restore the formating at the start of the PRINT line.
            
                If part_line > 0 Then
                    c_h_p(0) = c_h_p(0) + a_spac - spc_sz   'Add space at the START
                End If                                      'of a continued line.
                
                If com(0, sect) <> "" Then 'Set up the initial formating.
                    form_font com(0, sect)
                End If
 
                char = clear_spaces(char, com(1, sect))
            
                If pp_flg = 0 Then 'make the Printer fonts transparent.
                    Printer.FontTransparent = True  'This is not sufficient,it also requires
                    iBKMode = SetBkMode(Printer.hDC, TRANSPARENT)   'the API call.
                End If
 
                Do Until sect = l_spac(0) And char > l_spac(1)  'loop until a line is completed.
                    t_char = Mid(com(1, sect), char, 1) 'Store the character.
 
                    If t_char = " " And alg = 3 Then    'Increment the horizontal position
                        c_h_p(0) = c_h_p(0) + a_spac    'for the next character after the
                    End If                              'space.
 
                    p_text pp_flg, t_char, c_h_p(0), vp 'Print it.
                
                    char = char + 1     'Increment the character count.
  
                    If char > Len(com(1, sect)) Then 'this is the end of this block, so
                        If sect < l_spac(0) Then    'there's more to be done, so
                            sect = sect + 1         'move to the next section.
                            form_font com(0, sect)  'set the formatting and
                            char = 1                'reset the character position.
                        End If
                    End If
                Loop 'to PRINT/PREVIEW the line.
            
                If print_depth > max_p_l + t_marg + f_marg Then 'the page is full.
                    MsgBox "The text is longer than a PAGE.", vbOKOnly + vbCritical, "'add_f_text' ERROR"
                    p_l = 0
                    p_d = print_depth
                    Exit Sub
                End If
                
                part_line = 0   'Clear any part line value for subsequent lines.

                store_format    'Store the formating at the end of the PRINT line for the next CHECK.
            
                st_sc = l_spac(0): st_ch = l_spac(1) + 1    'Reset the starting positions for the next line.

                If sect < coms Then 'there are still lines to print, so
                    terminate   'terminate the printing as this is a complete line.
                Else            'Otherwise, this is the last line, so
                    done = True 'flag for the end.
                End If
            End If 'there are no spaces.
        Loop 'until ALL lines done.

        If tm Then
            terminate       'Terminate the printing according to the parameter and
            p_l = 0         'return a zero partial line value
        Else                                        'Otherwise,
            p_l = l_spac(2) - l_marg + 2 * spc_sz   'set the partial line value to the
        End If                                      'EOL value plus some space.
        
        print_depth = print_depth + vp
        
        p_d = print_depth  'Return the current print depth.
    End If 'not canceled.
  
End Sub

'Routine to clear out any leading spaces in a line.
'Function name: clear_spaces(ch, tx)
'Parameters:    ch, the character number,
'               tx; the text.

Private Function clear_spaces(ch, tx)
 
    Dim cha As Integer
    Dim t_c As String
    
    cha = ch
    
    t_c = Mid(tx, cha, 1)   'Remove any leading spaces from a line
    Do Until t_c <> " "     'by incrementing the character number
        cha = cha + 1       'to point to the first non-space character.
        t_c = Mid(tx, cha, 1)
    Loop
 
    clear_spaces = cha
    
End Function

'Routine to terminate a print/display line.
'Procedure name:    terminate()

Public Sub terminate()

    If pp_flg = 0 Then
        Printer.Print                   'Terminate the line, if flagged.
        print_depth = Printer.CurrentY  'Store the vertical position.
    Else
        g_obj.Print                     'Terminate the line, if flagged.
        print_depth = g_obj.CurrentY    'Store the vertical position.
    End If
 
End Sub

'Routine to divide up a string into TEXT and FORMATING characters.
'Function name: parse_text(ttx)
'Parameter:     ttx;    the text string.

Private Sub parse_text(ttx)

    Dim pt, l_m, r_m As Integer
 
    pt = 1                  'Initialise the character pointer,
    coms = 0                'the command string count and
    ReDim com(1, coms + 1)  'dimension the command storage array.
    
    l_m = InStr(pt, ttx, fl_char)   'Find the first left marker, '\', and
    remove_marker ttx, l_m           'check that it isn't just a \ in the text.

    Select Case l_m
        Case Is > 1 'there's text before the first code block, so
            com(0, 0) = ""                  'store no code and
            com(1, 0) = Left(ttx, l_m - 1)  'the initial section of text.
            coms = 1
        Case 0  'there aren't any formating codes, so
            com(0, 0) = ""  'store no code and
            com(1, 0) = ttx 'all the text.
    End Select
 
    If l_m > 0 Then
        Do Until l_m = 0
            If l_m <> 0 Then 'there may be formating commands, so
                r_m = InStr(l_m, ttx, fr_char)   'Find a right marker , '/'.
                If r_m <> 0 And r_m <> l_m + 1 Then 'there are  formating commands, so
                    ReDim Preserve com(1, coms + 1)                 'redimension the command array,
                    com(0, coms) = Mid(ttx, l_m + 1, r_m - l_m - 1) 'store the commands and
                    pt = r_m + 1                    'Update the pointer and
                    l_m = InStr(pt, ttx, fl_char)   'find the next left marker, '\'.
                    If l_m = 0 Then 'there isn't another left marker, so
                        com(1, coms) = Right(ttx, Len(ttx) - r_m) 'store the rest of the text.
                    Else                                        'or
                        remove_marker ttx, l_m
                        com(1, coms) = Mid(ttx, r_m + 1, l_m - r_m - 1) 'Store the next section of text.
                    End If
                    coms = coms + 1     'Increment the command count.
                End If
            End If
        Loop
        coms = coms - 1 'Adjust the command count.
    End If
 
    ss_set = False      'Clear the Super/Subscript set flag.

End Sub

'Routine to check for multiple left-markers in the text and remove one of them.
'Procedure name:    remove_marker(tx, mp)

Private Sub remove_marker(tx, mp)

    Do Until Mid(tx, mp + 1, 1) <> fl_char  'Loop until the next character isn't a marker.
        tx = shorten_text(tx, mp)        'remove one of the '\' characters and
        Do Until Mid(tx, mp + 1, 1) <> fl_char 'loop
            mp = mp + 1                   'until a non-'\' character is found.
        Loop
        mp = InStr(mp + 1, tx, fl_char) 'find the next left marker, '\'.
    Loop

End Sub

'Routine to remove a code indicator from the text to leave a single character.
'Function name: shorten_text(pt)
'Parameter:     pt, the position of the character to be removed.

Private Function shorten_text(txtx, pt)

    Dim ln As Integer
    
    ln = Len(txtx)  'Store the length of the string.
    
    shorten_text = Left(txtx, pt - 1) + Right(txtx, ln - pt)
    
End Function

'Routine to store the current text formating for possible replacement.
'Procdeure Name:    store_format

Private Sub store_format()
 
    'Only save the PRINTER data.
    s_f_n = Printer.font.Name
    s_f_z = Printer.font.Size
    s_f_b = Printer.font.Bold
    s_f_i = Printer.font.Italic
    s_f_u = Printer.font.Underline
    s_f_s = Printer.font.Strikethrough
    s_f_c = Printer.ForeColor
 
End Sub

'Routine to replace stored text formatting data.
'Procedure name:    re_store_format

Private Sub re_store_format()

    'Restore the PRINTER font data.
    Printer.font.Name = s_f_n
    Printer.font.Size = s_f_z
    Printer.font.Bold = s_f_b
    Printer.font.Italic = s_f_i
    Printer.font.Underline = s_f_u
    Printer.font.Strikethrough = s_f_s
    Printer.ForeColor = s_f_c
    
    'Restore the DISPLAY OBJECT font data.
    g_obj.font.Name = s_f_n
    g_obj.font.Size = s_f_z * ratio * p_zoom
    g_obj.font.Bold = s_f_b
    g_obj.font.Italic = s_f_i
    g_obj.font.Underline = s_f_u
    g_obj.font.Strikethrough = s_f_s
    g_obj.ForeColor = s_f_c
 
End Sub

'Routine to set the horizontal position for the start of a line
'in terms of the required alignment and the text length.
'Function name: aline(al, h, txt_l)
'Parameters:    al;     the alignment code,0, 1, 2, or 3,
'               h;      the horizontal offset,
'               txt_l;  the length of the formated text,
'               prt_l;  the partial line value.

Private Function aline(al, h, txt_l, Optional prt_l = 0)

    Dim hpl As Single
 
    canceled = False    'Clear the CANCELed flag and
    hpl = 0             'set default return value.
    
    If prt_l > 0 And al >= 1 And al <= 2 Then
        MsgBox "You can't have a partial line with CENTRE or RIGHT alingments.", vbOKOnly + vbCritical, "ALIGNMENT ERROR"
        canceled = True
        hpl = 0
    End If
 
    If Not canceled Then
        Select Case al 'All alignments include the left margin.
            Case 0, 3 'is LEFT or FULLY JUSTIFIED.
                If prt_l > 0 Then
                    hpl = l_marg + prt_l + spc_sz   'Use in the partial line value with a single space.
                Else                    'Otherwise,
                    hpl = l_marg + h    'Start at the left margin plus any offset.
                End If
            Case 1 'is CENTRED.
                hpl = l_marg + h + (l_width - txt_l) / 2
            Case 2 'is RIGHT.
                hpl = l_marg + h + l_width - txt_l
        End Select
   
        If pp_flg = 0 Then      'This is for Printing, so
            hpl = hpl - LRgap   'remove the Printer Width dead space.
        End If 'printing.
    End If 'not canceled
                 
    aline = hpl

End Function

'Routine to print a text character, (used for FULL JUSTIFICATION).
'Procedure name:    p_text(fl, txx, hh, vv)
'Parameters:        fl,     the PRINT/PREVIEW flag,
'                   txx;    the text character,
'                   hh;     the horizontal position,
'                   vv;     the vertical offset.

Private Sub p_text(fl, txx, ByRef hh, Optional vv = 0)
 
    Dim cw, hs, hm, bck As Single
    
    If b_s_f Then 'this character needs to be backspaced.
        cw = Printer.TextWidth(txx) 'Get the width of this character,
        bck = (ccw + cw) / 2        'find the backspace step and
        hm = (ccw - cw) / 2         'the value to restore the next character position.
    Else
        bck = 0
        hm = 0
    End If
    hs = hh - bck   'Store the entry value.
 
    Select Case fl
        Case 0 'is print.
            Printer.CurrentX = hs               'Position text at specified X position and
            Printer.CurrentY = print_depth + vv 'Y position.
            Printer.Print txx;
            hh = hs + hm + Printer.TextWidth(txx)  'Return the horizontal position via the parameter.
        Case 1 'is preview
            g_obj.CurrentX = hs                  'Position text at specified X position and
            g_obj.CurrentY = (print_depth + vv)  'Y position.
            g_obj.Print txx;
            hh = hs + hm + g_obj.TextWidth(txx)     'Return the horizontal position via the parameter.
    End Select
  
    b_s_f = False: bck = 0: hm = 0  'Clear the flag and storage values.

End Sub

'Routine to find the size of characters and return the cumulative horizontal position.
'This is used by the first stage of 'add_f_text' to get the line splitting values.
'Procedure name:    check_text(txx, ByRef hpp, ByRef hvv)
'Parameters:        txx;    the text character,
'                   hpp;    the current horizontal PRINTER position,
'                   hvv;    the current horizontal Object position.

Private Sub check_text(txx, ByRef hpp, ByRef hvv)
  
    If b_s_f Then 'this character needs to be backspaced.
        'so ignore it all together and
        'leave the horizontal values unchanged.
    Else
        hpp = hpp + Printer.TextWidth(txx)  'Return the values updated by
        hvv = hvv + g_obj.TextWidth(txx)    'the width of the character.
    End If
 
    b_s_f = False

End Sub

'Routine to add a graphic to the page.

'The position values must be given in units from LEFT and TOP margins but the graphic
'WIDTH and HEIGHT can be given either in units or as a NEGATIVE FRACTION of the page
'width and height.

'The default value of zero for the width means that the source width is maintained.
'The default value of zero for the height means that the proportion of the graphic is
'maintained. i.e. zeroes for 'gw' and 'gh' mean that the graphic is displayed at its
'original size.
'If 'hh' is defaulted to -1, the graphic will be CENTRED on the page.
'If 'vv' is defaulted to zero, the graphic is printed one line space below the current
'PAGE DEPTH.

'Procedure name:    add_graphic(g_path As String, Optional gw As Single = 0,
'                   Optional gh As Single = 0, Optional hh As Single = -1,
'                   Optional vv As Single = 0,Optional obj As Variant)

'Parameters:        path;       graphic file name plus full path,
'                   gw;         graphic width in units or as a NEGATIVE fraction of the
'                               page width,
'                   gh;         graphic height in units or as a NEGATIVE fraction of
'                               the page height,
'                   hh;         relative horizontal position or default, (-1), to centre,
'                   vv;         relative vertical position or default, (0), for current
'                               depth plus a single line space.

Public Sub add_graphic(g_path As String, Optional gw As Single = 0, Optional gh As Single = 0, _
                       Optional hh As Single = -1, Optional vv As Single = 0)

    Dim a_dp, s_w, s_h, hp, vp, gr_w, gr_h As Single
    
    a_dp = max_p_l - print_depth - n_l_h     'Store the available depth left on the page.
   
    'Load the graphic.
    pic.Picture = LoadPicture(g_path)    'Load the graphic into the Picture box and
    s_w = pic.Width                      'store the width and
    s_h = pic.Height                     'height.
 
    Select Case True 'Set the graphic WIDTH.
        Case gw = 0     'This is the default, so keep source width.
            gr_w = s_w  'set the width to that of the source.
        Case gw < 0 And gw > -1     'This is a negative fraction, so
            gr_w = l_width * (-gw)  'set the graphic width in terms of the line length.
        Case gw < -1      'is an ERROR.
            MsgBox CStr(gw) + " is an unacceptable width.", vbOKOnly + vbCritical, "GRAPHIC IMPORT ERROR"
            canceled = True
            Exit Sub
        Case Else       'Otherwise,
            gr_w = gw   'set the width to the parameter.
    End Select
    
    If gr_w > l_width Then  'there's no room for the graphic, so
        MsgBox "The original graphic is too wide for the page.", vbOKOnly + vbCritical, "GRAPHIC IMPORT ERROR"
        canceled = True
        Exit Sub
    End If

    Select Case True 'Set the graphic HEIGHT.
        Case gh = 0                 'This is the default, so
            gr_h = gr_w * s_h / s_w 'set the graphic height in proportion to the source size.
        Case gh < 0 And gh > -1     'This is a negative fraction, so
            gr_h = max_p_l * (-gh)  'set the graphic height in terms of the page length.
        Case gh < -1 'is an ERROR
            MsgBox CStr(gh) + " is an unacceptable height.", vbOKOnly + vbCritical, "GRAPHIC IMPORT ERROR"
            canceled = True
            Exit Sub
        Case Else       'Otherwise,
            gr_h = gh   'set the height to the parameter.
    End Select
    
    If gr_h > a_dp Then 'there's no room for the graphic, so
        MsgBox "The selected height is too tall for the page.", vbOKOnly + vbCritical, "GRAPHIC IMPORT ERROR"
        canceled = True
        Exit Sub
    End If
  
    Select Case hh  'Set the LEFT position.
        Case -1                                 'This is the default, so
            hp = l_marg + (l_width - gr_w) / 2  'centre the graphic.
        Case Else               'Otherwise,
            hp = l_marg + hh    'set from the left margin.
    End Select

    Select Case vv  'Set the TOP position.
        Case 0                          'This is the default, so
            vp = print_depth + n_l_h    'set the graphic a line space down from the current print depth.
        Case Else                       'Otherwise,
            vp = vv + t_marg            'set from the top margin.
    End Select
        
    'DRAW the graphic
    If pp_flg = 0 Then
        Printer.PaintPicture pic.Image, hp, vp, gr_w, gr_h  'Add the picture to the PRINT page,
    Else
        g_obj.PaintPicture pic.Image, hp, vp, gr_w, gr_h      'Add the picture to the screen object.
    End If
    
    hh = hp + gr_w                  'Return the horizontal position via the 'hh' parameter.
    
    print_depth = vp + gr_h + n_l_h 'Reset the print depth with a line space and
    vv = print_depth                'return the value via the 'vv' parameter.
    
End Sub

'Routine to set in_line text formating via the formating codes..
'Procedure name:    form_font(txx)
'Parameter:         txx;    a text string of formating characters.
'

Private Sub form_font(txx)
 
    Dim t_l, j, pt, num As Integer
    Dim p_char, n_c, n_t As String
    
    t_l = Len(txx)      'Store the text length and
    
    If t_l = 0 Then     'there are no codes for this block, so
        re_set_font     'reset the font parameters and
        ss_set = False  'clear the Super/Sub flag.
    Else
        For j = 1 To t_l
            p_char = UCase(Mid(txx, j, 1))
            
            'Analyse the letter codes.
            Select Case p_char
                Case "K"    'Backspace, (this is first as it needs the previous format intact).
                    ccw = Printer.TextWidth(t_char) 'Store the current character size and
                    b_s_f = True                    'flag to backspace the next character.
                Case "B"    'Set Bold.
                    Printer.font.Bold = True        'Set the printer font.
                    g_obj.font.Bold = True   'Set the Picture Box font.
                Case "I"    'Set Italic.
                    Printer.font.Italic = True      'Set the printer font.
                    g_obj.font.Italic = True 'Set the Picture Box font.
                Case "U"    'Set Underline.
                    Printer.font.Underline = True   'Set the printer font.
                    g_obj.font.Underline = True   'Set the Picture Box font.
                Case "T"    'Set StrikeThrough.
                    Printer.font.Strikethrough = True   'Set the printer font.
                    g_obj.font.Strikethrough = True   'Set the Picture Box font.
                Case "Y"    'Change to "Symbol" font.
                    Printer.font.Name = sym_font    'Set the fonts to the
                    g_obj.font.Name = sym_font      'stored 'Symbol' font.
                Case "W"    'Change to "WingDings" font.
                    Printer.font.Name = wd_font     'Set the fonts to the
                    g_obj.font.Name = wd_font       'stored 'WingDings' font.
                Case "D"    'Change back to default font.
                    Printer.font.Name = f_name  'Reset the fonts to the
                    g_obj.font.Name = f_name    'original font.
                Case "S"    'Set Superscript.
                    If Not ss_set Then 'No S/P flag is set, so
                        Printer.font.Size = Printer.font.Size * 0.65 'Change the font sizes.
                        g_obj.font.Size = g_obj.font.Size * 0.65
                        prntd = print_depth 'Store the current printing depth.
                        print_depth = print_depth - l_ht * 0.3  'set the position and
                        ss_set = True   'flag that Super or Subscript is set.
                    Else                                    'Otherwise,
                        print_depth = prntd - l_ht * 0.3    'set the position from the stored depth.
                    End If
                Case "P"    'Set Subscript.
                    If Not ss_set Then 'No S/P flag is set, so
                        Printer.font.Size = Printer.font.Size * 0.65
                        g_obj.font.Size = g_obj.font.Size * 0.65
                        prntd = print_depth 'Store the current printing depth.
                        print_depth = print_depth + l_ht * 0.1   'set the position and
                        ss_set = True   'flag that Super or Subscript is set.
                    Else                                    'Otherwise,
                        print_depth = prntd + l_ht * 0.1 '3 'set the position from the stored depth.
                    End If
                Case "N"   'Return to normal.
                    re_set_font
                    ss_set = False
                Case "Q" 'This should be a QB colour code?
                    pt = j + 1
                    n_c = ""
                    Do Until Not IsNumeric(Mid(txx, pt, 1))
                        n_c = n_c + Mid(txx, pt, 1)
                        pt = pt + 1
                    Loop
                    j = pt - 1      'Reset the loop variable.
                    num = Val(n_c)  'Get the number.
                    If num >= 0 And num < 16 And Not check Then 'it's a QB code, so
                        g_obj.ForeColor = QBColor(num)
                        Printer.ForeColor = QBColor(num)
                    End If
                    If num < 0 Or num >= 16 Then
                        MsgBox "Code 'Q" + n_c + "' isn't a valid QB code.", vbOKOnly, "FORMAT CODE ERROR"
                        canceled = True
                        Exit Sub
                    End If
                Case Else
                    'If this is a numeric it's part of a font SIZE.
                    If IsNumeric(p_char) Then
                        pt = j + 1                              'Check the next characters
                        n_t = p_char                            'to get
                        Do Until Not IsNumeric(Mid(txx, pt, 1)) 'all of the digits.
                            n_t = n_t + Mid(txx, pt, 1)
                            pt = pt + 1
                        Loop
                        num = Val(n_t)
                        If num > 6 Then
                            Printer.font.Size = num    'Set the font sizes.
                            g_obj.font.Size = num * ratio * p_zoom
                        Else
                            MsgBox "Font sizes must be greater than 6", vbOKOnly, "FORMAT CODE ERROR"
                        End If
                        j = pt - 1            'Reset the loop variable.
                    Else
                        MsgBox "Code '" + p_char + "' not recognised.", vbOKOnly, "FORMAT CODE ERROR"
                        canceled = True
                        Exit Sub
                    End If
            End Select
        Next
    End If
 
    l_ht = Printer.TextHeight("I")  'Reset the height of a character, (line width), and
    spc_sz = Printer.TextWidth(" ") 'the size of a space in the current font size.
    
    If l_ht > n_l_h Then    'update the New Line Height.
        n_l_h = l_ht
    End If
        
End Sub

'Routine to reset text formating to the defaults.
'Procedure name:    re_set_font()


Private Sub re_set_font()

    pp_font             'Reset the font parameters to the stored values and
    
    If ss_set Then
        print_depth = prntd 'store the print depth.
    End If
    
End Sub

'Routine to add a header or footer to a page.
'Procedure name:    head_foot(flg, Optional ltx As String = "", Optional ctx As String = "",
'                             Optional rtx As String = "", Optional f_nm As String = "",
'                             Optional f_sz As Single = 0, Optional f_mb As Boolean = False)
'Parameter:         flg;    flag, 0 = Header, 1 = Footer,
'                   ltx;    LEFT header text,
'                   ctx;    CENTER header text,
'                   rtx;    RIGHT header text,
'NB No check is made to see if any of the sections overlap!
'                   f_nm;   the font name,
'                   f_sz;   the font size,
'                   f_mb;   font Bold flag.

Public Sub head_foot(flg, Optional ltx As String = "", Optional ctx As String = "", _
                     Optional rtx As String = "", Optional f_nm As String = "", _
                     Optional f_sz As Single = 0, Optional f_mb As Boolean = False)
 
    Dim j As Integer
    Dim p_fsz, pd, ppd As Single
    Dim htx(2), p_fn, htxt As String
    Dim p_fb As Boolean

    htx(0) = ltx: htx(1) = ctx: htx(2) = rtx   'Store the header texts in array.
    
    If f_nm <> "" Then
        p_fn = f_name       'Store the OLD name and
        set_font f_nm, 0    'set the NEW name.
    End If

    If f_sz <> 0 Then
        p_fsz = f_size      'Store the OLD size and
        set_font f_sz, 1    'set the NEW size.
    End If
    
    p_fb = f_bold       'Store the OLD Bold flag, (even if it may not be different),
    set_font f_mb, 2    'and set the NEW Bold status.
    
    If flg = 0 Then     'Set the printing depth.
        pd = h_marg
    Else
        pd = (t_marg + max_p_l + f_marg)
    End If
    
    For j = 0 To 2
        Select Case LCase(htx(j))
            Case ""     'There is no text for this section.
                htxt = ""
            Case "sd"   'This is to be the DATE in short form.
                htxt = Format(Now, form_sd)
            Case "ld"   'This is to be the DATE in LONG form.
                htxt = super_date(Now, True)  'Format the date with superscript.
            Case "pa"   'This is to be the page number in ARABIC numerals.
                htxt = "Page " + CStr(p_a_num)
            Case "pr"   'This is to be the page number in ROMAN numerals.
                htxt = FormatRoman(p_a_num)
            Case Else   'This is just the text.
                htxt = htx(j)
        End Select
        
        ppd = pd
        add_f_text htxt, ppd, , j 'Display the Header text.
    Next
    
    'Restore the original font parameters.
    If f_nm <> "" Then
        set_font p_fn, 0    'Restore the font name.
    End If
    
    If f_sz <> 0 Then
        set_font p_fsz, 1   'Restore the font size.
    End If
    
    set_font p_fb, 2    'Restore the original Bold status.
 
    terminate               'Terminate the text and then
    print_depth = t_marg    'set initial print position for the next page to the top margin.
    
End Sub

'Routine to find the start position to centre a piece of text.
'The Print/Preview flag is ignored as the position is found in terms of
'PRINTING text which is scaled later.

'Function name:     centre(tx)
'Parameter:         tx;    the text,

Private Function centre(tx)

    Dim l_txt As Long

    l_txt = Printer.TextWidth(tx)  'Get the text width.
 
    centre = (l_width - l_txt) / 2    'Return the X-position.
 
End Function

'Routine to create a 'super' date string with superscript 'st', 'nd', 'rd' and 'th'.
'Function name: super_date(dt, f)
'Parameter:     dt; the date.
'               f;  the formated text flag, (True = add custom formating flags).
'                   N.B. these will only work if 'add_f_text' is used.

Public Function super_date(dt As Date, f)

    Dim str1, str2, str3, super As String
    Dim d_n, f_d As Integer
    
    str1 = Format(dt, "mmmm")   'Store the month.
    str2 = Format(dt, "d")      'Get the day number,
    f_d = Val(str2) \ 10        'its first digit and
    d_n = Val(str2) Mod 10      'convert it to 0 - 9.
    str3 = Format(dt, "yyyy")   'Store the year.
 
    Select Case d_n             'Use 'd_n' to select the superscript text.
        Case 0, 4, 5, 6, 7, 8, 9
            If f Then
                super = "\s/th\n/"
            Else
                super = " th"
            End If
        Case 1
            If f Then
                super = "\s/st\n/"
            Else
                super = " st"
            End If
        Case 2
            If f Then
                super = "\s/nd\n/"
            Else
                super = " nd"
            End If
        Case 3
            If f Then
                super = "\s/rd\n/"
            Else
                super = " rd"
            End If
    End Select
    
    If f_d = 1 Then             'Over-ride 'st', 'nd' & 'rd'
        If f Then               'if it's 11th, 12th or 13th.
            super = "\s/th\n/"
        Else
            super = " th"
        End If
    End If
    
    super_date = str1 + " " + str2 + super + ", " + str3   'Return the date string.
    
End Function

'Routine to set the page object's colour.
'Procedure name:    set_p_col(p_c)
'Parameter:         p_c;    the RGB colour value, defaults to 0 = global colour.

Public Sub set_p_col(Optional p_c As Long = 0)

    If p_c = 0 Then     'If this is the default value,
        p_c = p_col     'set the colour to 'p_col', defined by PPpages.
    End If
    
    g_obj.BackColor = p_c   'Set the 'paper' colour.
    
End Sub

'Routine to set the page object.
'Procedure name:    set_object(obj)
'Parameter:         obj, defaults to none = Preview Box

Public Sub set_object(Optional obj As Variant)

    If IsMissing(obj) Then  'If there is no value then
        Set g_obj = f_pic   'set the global object to 'f_pic', the Preview Box.
    Else                    'Otherwise,
        Set g_obj = obj     'set it to the paramater object.
    End If

End Sub

'Formats a number as a Roman numeral.
'Author: Christian d'Heureuse (www.source-code.biz)

'Function name: FormatRoman(ByVal n As Integer) As String
'Parameter:     n;  the integer to be formated.

Private Function FormatRoman(ByVal n As Integer) As String

    Dim i, p As Integer
    Dim s As String
    
    i = Abs(n)
    
    Const r = "ivxlcdm" 'Lower case Roman symbols.
    
    For p = 1 To 5 Step 2
        Dim d As Integer
        d = i Mod 10: i = i \ 10
        Select Case d   'Format a decimal digit
            Case 0 To 3
                s = String(d, Mid(r, p, 1)) & s
            Case 4
                s = Mid(r, p, 2) & s
            Case 5 To 8
                s = Mid(r, p + 1, 1) & String(d - 5, Mid(r, p, 1)) & s
            Case 9
                s = Mid(r, p, 1) & Mid(r, p + 2, 1) & s
        End Select
    Next
    
    s = String(i, "m") & s  'Format thousands.
  
    FormatRoman = s

   End Function

'Routine to check the current alignment of text printing.
'This isn't used by the Class but is useful to have around.
'[VB defaults are LEFT and TOP].

'Procedure name:    check_align(obj)
'Parameter:         obj;    the qualifying object being checked, Form, Picture Box, Printer, etc.

Private Sub check_align(obj As Object)

    Dim refpoint As Long
    
    refpoint = GetTextAlign(obj.hDC) 'Get the text alignment setting of the object.
 
    If (refpoint And TA_CENTER) = TA_CENTER Then
        MsgBox "Text will be displayed centered horizontally."
    ElseIf (refpoint And TA_RIGHT) = TA_RIGHT Then
        MsgBox "Text will be displayed right-justified."
    Else  'Assume ref. point on left edge of block.
        MsgBox "Text will be displayed left-justified."
    End If
    
    If (refpoint And TA_BASELINE) = TA_BASELINE Then
        MsgBox "Text will be lined to the BASE of the letters."
    ElseIf (refpoint And TA_BOTTOM) = TA_BOTTOM Then
        MsgBox "Text will be lined to the BOTTOM of the bounding box."
    Else
        MsgBox "Text will be lined to the TOP of the bounding box."
    End If
    
End Sub

