VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "PPpages"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'COLLECTION CLASS MODULE TO PROVIDE A COLLECTION
'OF PAGES OF PP_PAGE OBJECTS.

'There should be nothing here that is specific to any
'particular  project except that:-

'1 The error routines require a Public Boolean, 'canceled' in the main program.

'2 The routines use a main program procedure, 'create_page', to write/re-write
'  specific page data for PRINTING or PREVIEW.
'  (The alternative would be to have a specific 'page' object as a method. However, it was
'   felt that coding the necessary degree of flexibilty of font and format would be too
'   cumbersome).

'3 The printer selection procedures require that the Microsoft Common Dialog Control 6.0
'  (SP3), COMDLG.OCX, or the CDC replacement Class be included in the project components
'  with its 'Cancel' property set to 'True'.

'4 The FlatScrollBar controls require that the Microsoft Windows Common Controls - 2
'  6.0(SP4), MSCOMCT2.OCX, be included in the project components.

'5 The following icon files are required in the application folder:-
'       mag-plus.ico
'       mag-minus.ico

' Philip Bolt, 2005, 2006
'Last updated: 18/11/6

Option Explicit
    
'The Collection defined below is Private and used only to enumerate the pages.
'If its declaration is made Public, it will be available in the main program.

'The collection must hold the range of pages to be printed. If there is an option of
'ODD or EVEN page printing, this must be reflected in the numbering of the pages stored.
'e.g. a range of 3 odd pages starting at P.7 must be stored as 7, 9, 11 and 'p_inc'
'set to 2 so that the correct pages will be selected when the Scroll Bar is moved.

Private PPps As New PPpages         'PPpages' object.
Private PP_pgs As New Collection    'PPpages' collection.

Private Const n_ctrl As Integer = 6 'Number of controls.
Private Const n_parm As Integer = 9 'Number of size parameters.
Private Const z_f_v As Single = 1.5 'Zoom factor.

'Display parameters.
Private wd As Integer               'Screen width.
Private pic_ht As Integer           'Preview picture height.
Private pic_wd As Integer           'Preview picture width.
Private bar_width As Integer        'Scroll Bar width.

'Selected page data.
Private n_pges As Integer           'Number of pages.
Private p1 As Integer               'The first page number.
Private p_inc As Integer            'The page increment, 2 for ODD or EVEN pages.

'Current page data.
Private p_n As Integer              'Current page number.
Private flg As Integer              'Current Print/Preview flag.
Private z_m As Single               'Current zoom factor.

'Scroll Bar parameters.
Private p_rnge As Integer           'Range of scroll Bar values per page.
Private pvp As Integer              'Previous vertical Bar value.
Private v_dif As Integer            'Vertical Bar difference value.

'Paper size parameters
Private parm() As Integer           'Paramater storage array.
Private p_ht As Integer             'Printing page height.
Private p_wd As Integer             'Printing page width.
Private t_marg As Integer           'Top margin.
Private h_marg As Integer           'Header margin.
Private b_marg As Integer           'Bottom margin.
Private f_marg As Integer           'Footer margin.
Private l_marg As Integer           'Left margin.
Private r_marg As Integer           'Right margin.
Private max_p_l As Integer          'Maximum page length .
Private l_width As Integer          'Line width.
Private p_d_s As Integer            'Printer Width dead space.
Private p_size As Integer           'Page size, vbPRPSA4 = 9, vbPRPSA5 = 11.
Private p_orient As Integer         'Page orientation, vbPRORPortrait = 1, vbPRORLandscape = 2.
Private ratio As Single             'Page scaling factor.

Private path_way As String          'Application path string.

Private in_use As Boolean           'Scroll Bar in use flag.
Private setup As Boolean            'Data set-up flag.
Private zoom_on As Boolean          'Zoom on flag.
Private printer_selected As Boolean 'Printer has been selected.
Private no_v_bar As Boolean         'Vertical scroll bar hidden flag.

Private frm As Object               'The PREVIEW form.
Private ctrl() As Object            'PREVIEW form controls.
Private cdc As Object               'Common Dialog Control.

'Object constructor.
'Run when the object is created.

Private Sub Class_Initialize()

    If Not setup Then
        set_up
        setup = True
    End If
 
End Sub

'Routine to add a new page to the pages collection.
'Function name: Add(ByVal pge As integer)
'Parameter:     l_pge;   the LOCAL page number.
'               a_pge;   the ABSOLUTE page number.

Public Function Add(ByVal l_pge As Integer, Optional a_pge As Integer) As PPpage

    Dim j As Integer
    Dim ppp As New PPpage, PPp_f As PPpage      'Dimension the 'Page' objects.
 
    ppp.page_numbers l_pge, a_pge               'Store the page numbers.
 
    PP_pgs.Add ppp, CStr(ppp.page_number(0))    'Add the page to the collection with
                                                'the LOCAL page number as ID.
    For j = 0 To 9
        ppp.set_sizes j, parm(j)    'Transfer the integer page structure values and
    Next
    ppp.set_sizes 10, ratio         'the 'ratio' value to this page.
    
    n_pges = PP_pgs.Count           'Store the number of pages.
    
    Set PPp_f = PP_pgs.Item(1)      'Select the first page in the set and
    p1 = PPp_f.page_number(0)       'store the first page number.
    
    set_v_bar 0                 'Initialise the vertical Scroll Bar.
    
    Set Add = ppp           'Return the object.
 
End Function

'Routine to delete a page object from the collection.
'Procedure name:    Delete(ByVal pge As integer)
'Parameter:         pge;   the LOCAL page number.

Public Sub Delete(ByVal pge As Integer)

    Dim ppp As PP_page  'Dimension a 'Page' object and
    Dim j As Integer    'the loop variable.
    
    If n_pges > 0 Then      'There are some pages in the collection, so
        For j = 1 To n_pges 'check each one.
            If PP_pgs(j).page_number(0) = pge Then  'this is the one, so
                Set ppp = PP_pages(j)               'set the object variable to the page,
                PP_pgs.Remove j                     'remove it from the collection and
                Set ppp = Nothing                   'release the handle.
            End If
        Next
        n_pges = PP_pgs.Count   'Re-store the number of pages.
    End If
    
End Sub

'Routine to close down a PP_pages object.
'Procedure name:    Class_Terminate()

Private Sub Class_Terminate()

    Dim ppp As PP_page      'Dimension a 'Page' object.
    
    For Each ppp In PP_pgs  'Process each page in the collection,
        Set ppp = Nothing   'close it.
    Next

    setup = False           'Flag that data is NOT set-up and
    Set PPps = Nothing      'release the 'Pages' object handle.
    
    Unload frm

End Sub

'Routine to set up an enumerator for the collection.
'NewEnum must return the IUnknown interface of a
'collection's enumerator.

'This function is hidden, (Select Tools, Procedure Attributes, Select 'NewEnum'
'                          Advanced, Check 'Hide this member', set 'Procedure ID' to -4)

Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"

   Set NewEnum = PP_pgs.[_NewEnum]
   
End Function

'Routine to initialise the Pages data.
'Procedure name:    set_up

Private Sub set_up()
 
    Dim fn As String
    
    path_way = App.path + "\"   'Define the application path string.
    
    Set frm = preview
    
    Set cdc = frm.CommonDialog1             'Define the CD control.
    
    ReDim ctrl(n_ctrl)                      'Dimension the control array.
    Set ctrl(0) = preview.Picture1          'Preview 'Window' picture box
    Set ctrl(1) = preview.Picture2          'Preview 'Child' picture Box.
    Set ctrl(2) = preview.FlatScrollBar1    'Preview VERTICAL scroll bar
    Set ctrl(3) = preview.FlatScrollBar2    'Preview HORIZONTAL scroll bar.
    Set ctrl(4) = preview.Command1          'CLOSE
    Set ctrl(5) = preview.Image1(0)         'Mouse + magnifying glass image.
    Set ctrl(6) = preview.Image1(1)         'Mouse - magnifying glass image.

    wd = Screen.Width
    
    pic_ht = 0.9 * Screen.Height - 2 * bar_width 'Set the picture box height to leave a 200 top & bottom
    pic_wd = pic_ht / Sqr(2)    'margin and the width to be in A4 ratio.
    
    bar_width = 200             'Scroll Bar widths.
    
    On Error GoTo etrap         'Set error trap.
    
    ctrl(5).Picture = LoadPicture(path_way + "mag-plus.ico")   'Load the Zoom mouse icons.
    ctrl(6).Picture = LoadPicture(path_way + "mag-minus.ico")
    
    On Error GoTo 0     'Clear the 'On error' jump.
   
    ReDim parm(n_parm)
    
    zoom_on = False             'Clear the zoom,
    printer_selected = False    'Printer Selected and
    no_v_bar = False            'hidden V-bar flags.
    
    Exit Sub
    
etrap: 'ERROR TRAP
    error_report Err.Number, fn, "'set_up'"
    
End Sub

'Routine to remove a set of pages from the Pages collection.
'Procedure name:    mt

Public Sub mt()

    Dim p_cnt, j, lst As Integer, ppp As PPpage 'Dimension the variables.
    
    p_cnt = PP_pgs.Count    'Count the pages.
    
    If p_cnt > 0 Then 'there are pages, so
        For j = p_cnt To 1 Step -1  'process until all pages removed.
            Set ppp = PP_pgs(j)     'Set the 'PP_page' variable to the page,
            PP_pgs.Remove j         'remove it from the collection and
            Set ppp = Nothing       'delete it.
        Next
        
        printer_selected = False    'Clear the Printer Selected flag.
    End If 'there are pages.
    
End Sub

'Routine to define the page properties of a new PP_pages object.
'Property Set name: page_parms(ByVal p_sz As interger, ByVal p_or As interger,
'                              ByVal t_m As Integer, ByVal h_m As Integer,
'                              ByVal b_m As Integer, ByVal f_m As Integer,
'                              ByVal l_m As Integer, ByVal r_m As Integer,
'                              ByVal p_i As Integer)

'Parameters:    p_sz;   the PAPER SIZE, vbPRPSA4 = 9, vbPRPSA5 = 11,
'               p_or;   the PAGE ORIENTATION, vbPRORPortrait = 1, vbPRORLandscape = 2,
'               t_m;    the TOP margin,
'               h_m;    the HEADER margin,
'               b_m;    the BOTTOM margin,
'               f_m;    the FOOTER margin,
'               l_m;    the LEFT margin,
'               r_m;    the RIGHT margin,
'               p_i;    the page increment, 1 for ALL, 2 for ODD or EVEN.

Public Sub page_parms(ByVal p_sz As Integer, ByVal p_or As Integer, _
                                ByVal t_m As Integer, ByVal h_m As Integer, _
                                ByVal b_m As Integer, ByVal f_m As Integer, _
                                ByVal l_m As Integer, ByVal r_m As Integer, _
                                ByVal p_i As Integer)
   
    p_size = p_sz       'page size,
    p_orient = p_or     'page orientation,
    t_marg = t_m        'top margin,
    h_marg = h_m        'header margin,
    b_marg = b_m        'bottom margin,
    f_marg = f_m        'footer margin,
    l_marg = l_m        'left margin,
    r_marg = r_m        'right margin,
    p_inc = p_i         'Page increment.
    
    parm(2) = t_m: parm(3) = h_m: parm(4) = b_m 'Also store the values in the
    parm(5) = f_m: parm(6) = l_m: parm(7) = r_m 'parameter array.
    parm(8) = p_i
    
    check_paper_size p_orient, p_size  'Set the paper size parameters.
    
End Sub

'Set of property routines to allow access to page structure data.
'The item required is selected by the 'opt' parameter.
'Property name:  page_data(opt)

Public Property Get page_data(opt As Integer)

    Select Case opt
        Case 0
            page_data = max_p_l 'Maximum page length
        Case 1
            page_data = l_width 'Line width
        Case 2
            page_data = t_marg  'top margin.
        Case 3
            page_data = h_marg  'header margin,
        Case 4
            page_data = b_marg  'bottom margin.
        Case 5
            page_data = f_marg  'footer margin.
        Case 6
            page_data = l_marg  'left margin.
        Case 7
            page_data = r_marg  'right margin.
        Case 8
            page_data = ratio   'the Print/Preview ratio.
        Case 9
            page_data = p_d_s   'the Printer Width dead space.
        Case 10
            page_data = z_m     'the current Zoom factor.
        Case Else
            MsgBox "Options must be in the range 0 to 10", vbOKOnly + vbCritical, "'PPpages:page_data' ERROR"
            canceled = True
    End Select
'MsgBox page_data
End Property

'Routine to set the PAGE INCREMENT for a range of pages.
'Property name: set_pinc(pnc As integer)
'Parameter:     pnx;    the increment.

Public Sub set_pinc(pnc As Integer)
 
    p_inc = pnc 'Store the increment value.
 
End Sub

'Routine to set up the PRINT PREVIEW form and Picture Boxes, etc.
'Procedure name:    set_preview_form

Private Sub set_preview_form()
 
    With frm
        .Height = pic_ht + bar_width    'Set the form to the usable screen height and
        .Width = pic_wd + bar_width     'the width to the picture box width.
        .Top = 200
        .Left = (wd - pic_wd) / 2
        .Visible = False
    End With
    
    With ctrl(0)                    'This is the WINDOW picture box.
        .Height = pic_ht
        .ScaleHeight = pic_ht
        .Width = pic_wd
        .ScaleWidth = pic_wd
        .Top = bar_width            'Top is set down for the buttons.
        .Left = 0
        .Visible = False
    End With
 
    With ctrl(1)                    'This is the CHILD picture box.
        .Height = pic_ht
        .ScaleHeight = pic_ht
        .Width = pic_wd
        .ScaleWidth = pic_wd
        .Top = 0                    'The child box is placed relative to
        .Left = 0                   'its parent.
        .MousePointer = 99          'Set for a custom pointer and
        .MouseIcon = ctrl(5).Picture 'load the 'Mag+' icon.
        .Visible = False
    End With

    With ctrl(2)                   'This is the VERTICAL scroll bar.
        .Height = ctrl(0).Height
        .Width = bar_width
        .Top = ctrl(0).Top
        .Left = ctrl(0).Left + ctrl(0).Width
        .Visible = True
    End With
    
    With ctrl(3)                   'This is the HORIZONTAL scroll bar.
        .Height = bar_width
        .Width = ctrl(0).Width
        .Top = ctrl(0).Height + bar_width
        .Left = ctrl(0).Left
        .Visible = False            'Hide the horizontal bar initially.
    End With

End Sub

'Routine to check the selected paper size, generate a printer scale factor and
'set up the PRINT PREVIEW form.
'Procedure name:    check_paper_size(rr, ps)
'Parameter:         rr; the orientation,
'                   ps; paper size.

Private Sub check_paper_size(rr, ps)

    Printer.Orientation = rr    'Set the specified orientation,
    Printer.PaperSize = ps      'paper size and
    Printer.ScaleMode = 1       'set the scale mode to twips and
 
    p_d_s = (Printer.Width - Printer.ScaleWidth) / 2    'Store the printer Width dead space.
    
    p_ht = Printer.ScaleHeight  'Store the vertical and
    p_wd = Printer.ScaleWidth   'horizontal printing sizes.

    max_p_l = p_ht - b_marg                 'Define the maximum allowable PRINTING depth.
    l_width = p_wd - r_marg - l_marg        'the maximum line length and
 
    parm(0) = max_p_l: parm(1) = l_width    'store them in the parameter array, with
    parm(9) = p_d_s                         'the dead space.
    
    set_preview_pic             'Set the size of the preview form and Picture box.
    
End Sub

'Routine to set the size of the Print Preview page.
'The Picture box height is a constant, 'pic_ht'.
'Procedure name:    set_preview_pic

Sub set_preview_pic()

    ratio = pic_ht / p_ht       'Define the Print Preview scale ratio.
 
    pic_wd = (ratio) * p_wd     'Set the width to be in paper size ratio.
    
    set_preview_form            'Adjust the preview form size.
    
End Sub

'Routine to modify the vertical Scroll Bar
'Procedure name:    change_v_bar(vpp)
'Parameter:         vpp;    the current vertical Bar value.

Public Sub change_v_bar(vpp)
 
    If Not in_use Then
        Dim pn, tp, p_pos, n_pos As Integer
    
        pvp = vpp                       'Store the value.
    
        check_arrows vpp, 2             'Check the arrows.
    
        pn = (vpp \ p_rnge) * p_inc + p1 'Get the indicated page number.
 
        If pn <> p_n Then  'this isn't the same page, so
            p_n = pn                   'Store the new page number and
            create_page flg, z_m, p_n  'display a new page.
           
            If vpp > pvp Then   'Check the direction of movement and
                tp = 0          'set the start of page
            Else                'position
                tp = -v_dif     'accordingly.
            End If
 
            ctrl(1).Top = tp   'reset the Top position.
        Else
            If zoom_on Then
                p_pos = vpp - (p_n - p1) * p_rnge       'Store the position value in page,
                n_pos = p_pos * v_dif / (p_rnge - 1)    'find the fraction of 'v_dif' concerned and
                ctrl(1).Top = -n_pos                    'move the Picture2 Top.
            End If
        End If
    End If 'not in use.
 
End Sub

'Routine to modify the horizontal Scroll Bar
'Procedure name:    change_h_bar(hpp)
'Parameter:         hpp;    the current horizontal Bar value.

Public Sub change_h_bar(hpp)

    check_arrows hpp, 3  'Check the arrows.
    
    ctrl(1).Left = -hpp 'Position the Picture2 box in the window.

End Sub

'Routine to check the scroll bar arrows.
'Procedure name:    check_arrows(pnn, sbn)
'Parameters:        pnn;    the position value, (usually the page number for the V Bar).
'                   sbn;    the Scroll Bar control value, 2=VERTICAL, 3=HORIZONTAL.

Private Sub check_arrows(pnn, sbn)

    Select Case pnn
        Case ctrl(sbn).Min
            ctrl(sbn).Arrows = cc2RightDown
        Case ctrl(sbn).Max
            ctrl(sbn).Arrows = cc2LeftUp
        Case Else
            ctrl(sbn).Arrows = cc2Both
    End Select
    
End Sub

'Routine to zoom in or out.
'Procedure name:    zoom

Public Sub zoom()

    If zoom_on Then 'the zoom flag is set, so
        zoom_out    'zoom out.
    Else            'Otherwise,
        zoom_in     'zoom in.
    End If
    
End Sub

'Routine to zoom IN on the display by a set amount.
'Procedute name:    zoom_in

Private Sub zoom_in()

    z_m = z_f_v         'Set the zoom factor and
    zoom_on = True      'flag that zoom is on.
        
    If no_v_bar Then    'this is a single page and the V-bar is hidden, so
        ctrl(2).Visible = True          'show the vertical bar,
        frm.Width = pic_wd + bar_width  'widen the form and
        no_v_bar = False                'clear the flag.
    End If
    
    ctrl(1).Cls                        'Clear the page and
    With ctrl(1)                       're-size the Picture box.
        .Height = ctrl(1).Height * z_m
        .Width = ctrl(1).Width * z_m
        .Top = 0
        .Left = 0
    End With
    
    frm.Height = frm.Height + bar_width 'Increase the height of the form to
                                        'take the horizontal bar.
    set_h_bar   'Set and display the Scroll Bars.
 
    set_v_bar (p_n - p1) * p_rnge       'Set the vertical Bar for the page.
    
    ctrl(1).MouseIcon = ctrl(6).Picture   'Load the 'Mag-' icon.

    create_page flg, z_m, p_n  'Display a new page.

End Sub

'Routine to zoom OUT on the display by a set amount.
'Procedute name:    zoom_out

Private Sub zoom_out()

    z_m = 1             'Set the zoom factor to unity and
    zoom_on = False     'flag that zoom is off.
        
    ctrl(1).Cls                     'Clear the page and
    With ctrl(1)                    're-size the Picture box.
        .Height = ctrl(0).Height    'to the original.
        .Width = ctrl(0).Width
        .Top = 0
        .Left = 0
    End With
    
    frm.Height = frm.Height - bar_width 'Reduce the height of the form.
    
    ctrl(3).Visible = False             'Hide horizontal Scroll Bar.
        
    ctrl(1).MouseIcon = ctrl(5).Picture 'Load the 'Mag+' icon.
    
    If n_pges = 1 Then
        hide_v_bar                  'Hide the V-bar for a single page.
    Else                            'Otherwise,
        set_v_bar 10 * (p_n - p1) '/ p_inc 're-set it.
    End If
    
    create_page flg, z_m, p_n  'display a new page.

End Sub

'Routine to set up the VERTICAL, scroll box.
'Procedure name:    set_v_bar(cvp)
'Parameter:         cvp;    current vertical position.

Private Sub set_v_bar(cvp)
 
    in_use = True                   'Set the in-use flag.
    p_rnge = Int(32767 / n_pges)    'Set the range of scroll bar values per page.
    
    If Not zoom_on Then     'Set the VERTICAL Scroll Bar for normal display.
        v_dif = 0           'Set value for Picture Box differences and
        p_rnge = 10         'the range of values per page.
            
        With ctrl(2)
            .Max = 10 * (n_pges - 1)
            .Min = 0
            .LargeChange = 10
            .SmallChange = 10
            .Value = cvp
        End With
    Else                    'Otherwise, set it for an enlarged display.
        v_dif = ctrl(1).Height - ctrl(0).Height     'Store the difference in height.
        With ctrl(2)
            .Max = n_pges * p_rnge - 3   'Approximate max. to 32767.
            .Min = 0
            .LargeChange = Int(.Max / n_pges)   '1 page a section.[was n_pges-1]
            .SmallChange = p_rnge / 100         '1/100th of a page.
            .Value = cvp
        End With
    End If
            
    pvp = cvp   'Set up the intial, stored value.
 
    in_use = False  'Clear the in-use flag.
    
End Sub

'Routine to set up the parameters of the HORIZONTAL Scroll Bar.
'Procedure name:      set_h_bar.

Public Sub set_h_bar()

    in_use = True
        With ctrl(3)
            .Max = ctrl(1).Width - ctrl(0).Width    'Set the maximum to the difference
            .Min = 0                                'in Picture widths.
            .SmallChange = Int(.Max / 10)
            .LargeChange = Int(.Max / 5)
            .Value = .Min
            .Visible = True
        End With
    in_use = False
    
End Sub

'Routine to show the preview form.
'Procedure name:    show_preview

Private Sub show_preview()

    If n_pges = 1 And Not zoom_on Then  'This is a single page, normal display, so
        hide_v_bar                      'hide the V-bar.
    End If
 
    With frm
        .Picture1.Visible = True    'Display both
        .Picture2.Visible = True    'Picture Boxes,
        .Visible = True             'show the form,
        .SetFocus                   'with Focus, and
    End With
 
End Sub

'Routine to hide the V-bar for a single page.
'Procedure name:    hide_v_bar

Sub hide_v_bar()

    ctrl(2).Visible = False 'Hide the vertical bar,
    frm.Width = pic_wd      'shrink the form and
    no_v_bar = True         'flag that this has been done.
        
End Sub

'Routine to close down the preview form.
'Procedure name:    close_preview

Public Sub close_preview()

    If zoom_on Then 'If zoom is on,
        zoom_out    'zoom out.
    End If
    
    frm.Visible = False 'Hide the PREVIEW form.
    
End Sub

'Routine to PRINT or PREVIEW a page.
'Procedure name:    print_it

Public Sub print_it(pn)
 
    Dim ppp As PPpage, indx As Integer
'MsgBox z_m
    p_n = pn    'Store the page number.
 
    indx = find_index((pn))     'Find the collection index for this page.
    
    If indx > 0 Then
        Set ppp = PP_pgs.Item(indx) 'select the page in the collection,
        flg = ppp.page_flag         'read the Print/Preview flag and
        z_m = ppp.page_data(10)     'the zoom factor.

        If flg = 0 Then
            Printer.EndDoc      'print the page.
        Else
            show_preview    'show the Preview form.
        End If
    End If 'index Ok, (Errors dealt with by 'find_index').

End Sub

'Routine to find the page for a specific LOCAL page number.
'Property name: find_page(pn)
'Parameter:     pn; the LOCAL page number.

Public Property Get find_page(pn As Integer)

    Dim p_cnt As Integer, j As Integer, pnn As Integer, indx As Integer
    
    indx = 0    'Initialise the index value.
    
    p_cnt = PP_pgs.Count                'Count the pages.
    If p_cnt > 0 Then
        For j = 1 To p_cnt                  'Check each page,
            pnn = PP_pgs(j).page_number(0)  'storing the LOCAL page number.
            If pnn = pn Then                'Compare this with the target number.
                indx = j                    'Store the index, if found, and
                j = p_cnt                   'terminate the loop.
            End If
        Next
    End If
    
    If indx = 0 Then 'there is an error.
        MsgBox "Page " + CStr(pn) + " isn't indexed!", vbOKOnly + vbCritical, "'find_page' ERROR"
        canceled = True
        Set find_page = Nothing
    Else
        Set find_page = PP_pgs(indx)
    End If
     
End Property

'Routine to find the collection index number for a specific LOCAL page number.
'Property name: find_index(pn)
'Parameter:     pn; the LOCAL page number.

Public Property Get find_index(pn As Integer)

    Dim p_cnt As Integer, j As Integer, pnn As Integer, indx As Integer
    
    indx = 0    'Initialise the index value.
    
    p_cnt = PP_pgs.Count                'Count the pages.
    If p_cnt > 0 Then
        For j = 1 To p_cnt                  'Check each page,
            pnn = PP_pgs(j).page_number(0)  'storing the LOCAL page number.
            If pnn = pn Then                'Compare this with the target number.
                indx = j                    'Store the index, if found, and
                j = p_cnt                   'terminate the loop.
            End If
        Next
    End If
    
    If indx = 0 Then 'there is an error.
        MsgBox "Page " + CStr(pn) + " isn't indexed!", vbOKOnly + vbCritical, "'find_index' ERROR"
        canceled = True
        find_index = 0
    Else
        find_index = indx
    End If
    
End Property

'Routine to return the current Pages collection object.
'Property name: page_col() As Collection

Public Property Get page_col() As Collection

    Set page_col = PP_pgs
    
End Property

'Routine to select a printer.
'Procedure name:    select_printer

Public Sub select_printer()

    If Not printer_selected Then
        canceled = False            'Clear the 'CANCELed' flag.
  
        cdc.DialogTitle = "PRINTER OPTIONS"  'Set up box to select printer properties.
        cdc.Flags = cdlPDDisablePrintToFile Or cdlPDHidePrintToFile Or cdlPDPrintSetup Or cdlPDReturnDC 'Or cdlPDReturnDC  'Or cdlPDReturnDefault
        cdc.PrinterDefault = True
    
        On Error GoTo userCancel    'Set the error, (CANCEL), jump label.
    
        cdc.ShowPrinter             'Display the 'Print' window.

        On Error GoTo 0             'Clear OnError jumps and
        printer_selected = True     'flag that a printer is selected.
    
        Printer.NewPage                 'Select a new page to allow over-rides.
        Printer.PaperSize = p_size      'Overide any selected values with the stored
        Printer.Orientation = p_orient  'parameters as these were selected at the start.
        Printer.KillDoc                 'Delete the blank, 'New', page.
    End If
    
    Exit Sub                    'quit the routine if not CANCELed.
    
userCancel:                     'The user CANCELed printing, so
 
    canceled = True             'set the 'CANCELed' flag.
   
End Sub

'Routine to select a printer. This version also sets the Excel workbook printer.
'Procedure name:    select_Excel_printer

Public Sub select_Excel_printer()

    If Not printer_selected Then
        canceled = False            'Clear the 'CANCELed' flag.
  
        cdc.DialogTitle = "PRINTER OPTIONS"  'Set up box to select printer properties.
        cdc.Flags = cdlPDDisablePrintToFile Or cdlPDHidePrintToFile Or cdlPDPrintSetup Or cdlPDReturnDC 'Or cdlPDReturnDC  'Or cdlPDReturnDefault
        cdc.PrinterDefault = True
    
        On Error GoTo userCancel    'Set the error, (CANCEL), jump label.
    
        cdc.ShowPrinter             'Display the 'Print' window.

        On Error GoTo 0             'Clear OnError jumps and
        printer_selected = True     'flag that a printer is selected.
    
        Printer.NewPage                 'Select a new page to allow over-rides.
        Printer.PaperSize = p_size      'Overide any selected values with the stored
        Printer.Orientation = p_orient  'parameters as these were selected at the start.
        Printer.KillDoc                 'Delete the blank, 'New', page.
    
        p_nme = Printer.DeviceName              'Get the printer name and use it to
        p_def = get_printer_string(p_nme)       'define the Excel printer string which is
        wbk.Application.ActivePrinter = p_def   'used for sheet printing.
        'NB 'wbk' is an ExcelWorkbook class object, defined as 'Public' in the main program.
    End If
    
    Exit Sub                    'quit the routine if not CANCELed.
    
userCancel:                     'The user CANCELed printing, so
 
    canceled = True             'set the 'CANCELed' flag.
   
End Sub

'Routine to get a suitable printer string for Excel.
'Function name: get_printer_string(p_nm)
'Parameter:     p_nm;   the printer name.

Private Function get_printer_string(p_nm)

    Dim p As Printer
    
    For Each p In Printers              'Check each printer and,
        If p.DeviceName = p_nm Then     'when the default is found,
            p_port = p.Port             'store the port name.
            Exit For
        End If
    Next
    
    get_printer_string = p_nm + " on " + p_port  'Return the printer and port.
    
End Function

'Error routine to report the various errors.
'Procedure name:    error_report(e_num, f_nm, r_nme)
'Parameter:         e_num;  the error number,
'                   f_nm;  the file name, (where applicable),
'                   r_nme; the routine name.

Private Sub error_report(e_num, f_nm, r_nme)

    Dim txt As String
    
    Select Case e_num
        Case 53 'is 'File not found'.
            txt = "File '" + f_nm + "' doesn't exist."
        Case Else
            txt = "Error # " & Str(Err.Number) & " was generated by " _
            & vbCrLf & Err.Source & vbCrLf & Err.Description
    End Select
    
    MsgBox txt, vbOKOnly + vbCritical, r_nme + " ERROR"
        
    Err.Clear       'Clear the error object.

    canceled = True 'Set the CANCELed flag.
    
End Sub

